"""Main conftest."""

import pytest

from synology_dsm.synology_dsm import SynologyDSM

from . import (
    VALID_HOST,
    VALID_HTTPS,
    VALID_PASSWORD,
    VALID_PORT,
    VALID_USER,
    SynologyDSMMock,
)


@pytest.fixture
def api() -> SynologyDSM:
    """Return a mock DSM 6 API."""
    return SynologyDSMMock(
        None,
        VALID_HOST,
        VALID_PORT,
        VALID_USER,
        VALID_PASSWORD,
        VALID_HTTPS,
    )


@pytest.fixture(
    params=[5, 6],
    ids=["DSM 5", "DSM 6"],
)
def dsm(request, api) -> SynologyDSM:
    """Return a mock DSM 6 API."""
    api.dsm_version = request.param
    return api


@pytest.fixture
def dsm_5(api) -> SynologyDSM:
    """Return a mock DSM 5 API."""
    api.dsm_version = 5
    return api


@pytest.fixture
def dsm_6(api) -> SynologyDSM:
    """Alias for api fixture."""
    return api


@pytest.fixture
def dsm_7(api) -> SynologyDSM:
    """Return a mock DSM 7 API."""
    api.dsm_version = 7
    return api



"""DSM 7 SYNO.Foto.Browse.Album data."""

DSM_7_FOTO_ALBUMS = {
    "data": {
        "list": [
            {
                "condition": {"folder_filter": [597], "user_id": 1},
                "create_time": 1674514932,
                "end_time": 1640958550,
                "id": 4,
                "item_count": 3,
                "name": "Album1",
                "owner_user_id": 1,
                "passphrase": "",
                "shared": False,
                "sort_by": "default",
                "sort_direction": "default",
                "start_time": 1601653233,
                "type": "condition",
                "version": 197920,
            },
            {
                "cant_migrate_condition": {},
                "condition": {},
                "create_time": 1668690757,
                "end_time": 1668538602,
                "freeze_album": False,
                "id": 1,
                "item_count": 1,
                "name": "Album2",
                "owner_user_id": 1,
                "passphrase": "",
                "shared": False,
                "sort_by": "default",
                "sort_direction": "default",
                "start_time": 1668538602,
                "temporary_shared": False,
                "type": "normal",
                "version": 195694,
            },
            {
                "cant_migrate_condition": {},
                "condition": {},
                "create_time": 1718658534,
                "end_time": 1719075481,
                "freeze_album": False,
                "id": 3,
                "item_count": 1,
                "name": "Album3",
                "owner_user_id": 7,
                "passphrase": "NiXlv1i2N",
                "shared": False,
                "sort_by": "default",
                "sort_direction": "default",
                "start_time": 1659724703,
                "temporary_shared": False,
                "type": "normal",
                "version": 102886,
            },
        ]
    },
    "success": True,
}

DSM_7_FOTO_ITEMS = {
    "success": True,
    "data": {
        "list": [
            {
                "id": 29807,
                "filename": "20221115_185642.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 1,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "29807_1668560967",
                        "unit_id": 29807,
                    }
                },
            },
            {
                "id": 29808,
                "filename": "20221115_185643.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 0,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "notready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "29808_1668560967",
                        "unit_id": 29808,
                    }
                },
            },
            {
                "id": 29809,
                "filename": "20221115_185644.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 1,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "notready",
                        "xl": "notready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "29809_1668560967",
                        "unit_id": 29809,
                    }
                },
            },
        ]
    },
}

DSM_7_FOTO_ITEMS_SHARED_ALBUM = {
    "success": True,
    "data": {
        "list": [
            {
                "id": 29807,
                "filename": "20221115_185645.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 7,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "29810_1668560967",
                        "unit_id": 29807,
                    }
                },
            },
        ]
    },
}

DSM_7_FOTO_SHARED_ITEMS = {
    "success": True,
    "data": {
        "list": [
            {
                "id": 77,
                "filename": "shared_1.jpg",
                "filesize": 1404758,
                "time": 1627062628,
                "indexed_time": 1628329471168,
                "owner_user_id": 0,
                "folder_id": 17,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "77_1628323785",
                        "unit_id": 77,
                    }
                },
            },
            {
                "id": 490,
                "filename": "shared_2.jpg",
                "filesize": 888192,
                "time": 1627062618,
                "indexed_time": 1628329516646,
                "owner_user_id": 0,
                "folder_id": 37,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "490_1628323817",
                        "unit_id": 490,
                    }
                },
            },
            {
                "id": 96,
                "filename": "shared_3.jpg",
                "filesize": 4903571,
                "time": 1626987559,
                "indexed_time": 1628329472531,
                "owner_user_id": 0,
                "folder_id": 18,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "96_1628323786",
                        "unit_id": 96,
                    }
                },
            },
        ]
    },
}

DSM_7_FOTO_ITEMS_SEARCHED = {
    "success": True,
    "data": {
        "list": [
            {
                "id": 12340,
                "filename": "search_1.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 1,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "12340_1668560967",
                        "unit_id": 12340,
                    }
                },
            },
            {
                "id": 12341,
                "filename": "search_2.jpg",
                "filesize": 2644859,
                "time": 1668538602,
                "indexed_time": 1668564550862,
                "owner_user_id": 1,
                "folder_id": 597,
                "type": "photo",
                "additional": {
                    "thumbnail": {
                        "m": "ready",
                        "xl": "ready",
                        "preview": "broken",
                        "sm": "ready",
                        "cache_key": "12341_1668560967",
                        "unit_id": 12341,
                    }
                },
            },
        ]
    },
}



"""DSM 7 SYNO.Core.ExternalDevice.Storage.USB data."""

DSM_7_CORE_EXTERNAL_USB_DS1821_PLUS_NO_EXTERNAL_USB = {
    "data": {"devices": []},
    "success": True,
}
DSM_7_CORE_EXTERNAL_USB_DS1821_PLUS_EXTERNAL_USB = {
    "data": {
        "devices": [
            {
                "dev_id": "usb1",
                "dev_title": "USB Disk 1",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "ntfs",
                        "filesystem": "ntfs",
                        "name_id": "usb1p1",
                        "partition_title": "USB Disk 1 Partition 1",
                        "share_name": "usbshare1-1",
                        "status": "normal",
                        "total_size_mb": 1024,
                        "used_size_mb": 10,
                    },
                    {
                        "dev_fstype": "vfat",
                        "filesystem": "FAT32",
                        "name_id": "usb1p2",
                        "partition_title": "USB Disk 1 Partition 2",
                        "share_name": "usbshare1-2",
                        "status": "normal",
                        "total_size_mb": 1024,
                        "used_size_mb": 0,
                    },
                ],
                "producer": "PNY",
                "product": "Flash Drive",
                "progress": "",
                "status": "normal",
                "total_size_mb": 122070,
            },
            {
                "dev_id": "usb2",
                "dev_title": "USB Disk 2",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "ext4",
                        "filesystem": "ext4",
                        "name_id": "usb2p1",
                        "partition_title": "USB Disk 2 Partition 1",
                        "share_name": "usbshare2",
                        "status": "normal",
                        "total_size_mb": 15138295,
                        "used_size_mb": 10899326,
                    }
                ],
                "producer": "Western Digital Technologies, Inc.",
                "product": "easystore 264D",
                "progress": "",
                "status": "normal",
                "total_size_mb": 15259648,
            },
            {
                "dev_id": "usb3",
                "dev_title": "USB Disk 3",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "vfat",
                        "filesystem": "FAT32",
                        "name_id": "usb3p1",
                        "partition_title": "USB Disk 3 Partition 1",
                        "share_name": "usbshare3-1",
                        "status": "normal",
                        "total_size_mb": 197,
                        "used_size_mb": 10,
                    },
                    {
                        "dev_fstype": "hfsplus",
                        "filesystem": "hfsplus",
                        "name_id": "usb3p2",
                        "partition_title": "USB Disk 3 Partition 2",
                        "share_name": "usbshare3-2",
                        "status": "normal",
                        "total_size_mb": 117020,
                        "used_size_mb": 13965,
                    },
                ],
                "producer": "SanDisk Corp.",
                "product": " SanDisk 3.2Gen1",
                "progress": "",
                "status": "normal",
                "total_size_mb": 117348,
            },
            {
                "dev_id": "usb4",
                "dev_title": "USB Disk 4",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "exfat",
                        "filesystem": "exfat",
                        "name_id": "usb4p1",
                        "partition_title": "USB Disk 4 Partition 1",
                        "share_name": "usbshare4",
                        "status": "normal",
                        "total_size_mb": 241397,
                        "used_size_mb": 1,
                    }
                ],
                "producer": "PNY",
                "product": "USB 3.0 FD",
                "progress": "",
                "status": "normal",
                "total_size_mb": 241408,
            },
            {
                "dev_id": "usb5",
                "dev_title": "USB Disk 5",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "vfat",
                        "filesystem": "FAT32",
                        "name_id": "usb5p1",
                        "partition_title": "USB Disk 5 Partition 1",
                        "share_name": "usbshare5-1",
                        "status": "normal",
                        "total_size_mb": 3886,
                        "used_size_mb": 0,
                    },
                    {
                        "dev_fstype": "vfat",
                        "filesystem": "FAT32",
                        "name_id": "usb5p2",
                        "partition_title": "USB Disk 5 Partition 2",
                        "share_name": "usbshare5-2",
                        "status": "normal",
                        "total_size_mb": 32,
                        "used_size_mb": 27,
                    },
                ],
                "producer": "SanDisk Corp.",
                "product": "Cruzer Titanium+",
                "progress": "",
                "status": "normal",
                "total_size_mb": 3919,
            },
            {
                "dev_id": "usb6",
                "dev_title": "USB Disk 6",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "ntfs",
                        "filesystem": "ntfs",
                        "name_id": "usb6p1",
                        "partition_title": "USB Disk 6 Partition 1",
                        "share_name": "usbshare6",
                        "status": "normal",
                        "total_size_mb": 715371,
                        "used_size_mb": 203585,
                    }
                ],
                "producer": "Western Digital Technologies, Inc.",
                "product": "My Passport Essential (WDBACY)",
                "progress": "",
                "status": "normal",
                "total_size_mb": 715372,
            },
            {
                "dev_id": "usb8",
                "dev_title": "USB Disk 8",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "unknown",
                        "filesystem": "",
                        "name_id": "usb8",
                        "partition_title": "USB Disk 8",
                        "share_name": "",
                        "status": "normal",
                        "total_size_mb": "",
                    }
                ],
                "producer": "Genesys Logic, Inc.",
                "product": "All-in-One Cardreader",
                "progress": "",
                "status": "normal",
                "total_size_mb": 29,
            },
            {
                "dev_id": "sdq",
                "dev_title": "SD Card 1",
                "dev_type": "sdCard",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "vfat",
                        "filesystem": "FAT32",
                        "name_id": "sdq1",
                        "partition_title": "SD Card 1 Partition 1",
                        "share_name": "SDKarte",
                        "status": "normal",
                        "total_size_mb": 1903,
                        "used_size_mb": 76,
                    }
                ],
                "producer": "Genesys Logic, Inc.",
                "progress": "",
                "status": "normal",
                "total_size_mb": 1903,
            },
            {
                "dev_id": "sds",
                "dev_title": "USB Disk 1",
                "dev_type": "usbDisk",
                "formatable": True,
                "partitions": [
                    {
                        "dev_fstype": "ntfs",
                        "filesystem": "ntfs",
                        "name_id": "sds1",
                        "partition_title": "USB Disk 1 Partition 1",
                        "share_name": "usbshare1",
                        "status": "usbbackup",
                        "total_size_mb": 953868,
                        "used_size_mb": 862289,
                    }
                ],
                "producer": "Iomega Corp.",
                "product": "Prestige Portable Hard Drive",
                "progress": "",
                "status": "usbbackup",
                "total_size_mb": 953869,
            },
        ]
    },
    "success": True,
}



"""DSM Utilization data."""

from __future__ import annotations

from typing import TypedDict

from synology_dsm.api import SynoBaseApi
from synology_dsm.helpers import SynoFormatHelper

CpuUtilization = TypedDict(
    "CpuUtilization",
    {
        "15min_load": int,
        "1min_load": int,
        "5min_load": int,
        "device": str,
        "other_load": int,
        "system_load": int,
        "user_load": int,
    },
)

MemoryUtilization = TypedDict(
    "MemoryUtilization",
    {
        "avail_real": int,
        "avail_swap": int,
        "buffer": int,
        "cached": int,
        "device": str,
        "memory_size": int,
        "real_usage": int,
        "si_disk": int,
        "so_disk": int,
        "swap_usage": int,
        "total_real": int,
        "total_swap": int,
    },
)

NetworkUtilization = TypedDict(
    "NetworkUtilization",
    {
        "device": str,
        "rx": int,
        "tx": int,
    },
)


class UtilizationDataType(TypedDict, total=False):
    """Data type."""

    cpu: CpuUtilization
    memory: MemoryUtilization
    network: list[NetworkUtilization]


class SynoCoreUtilization(SynoBaseApi[UtilizationDataType]):
    """Class containing Utilization data."""

    API_KEY = "SYNO.Core.System.Utilization"

    async def update(self) -> None:
        """Updates utilization data."""
        raw_data = await self._dsm.get(self.API_KEY, "get")
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data

    @property
    def cpu(self) -> CpuUtilization:
        """Gets CPU utilization."""
        return self._data["cpu"]

    @property
    def cpu_other_load(self) -> int:
        """Other percentage of the total CPU load."""
        return self.cpu["other_load"]

    @property
    def cpu_user_load(self) -> int:
        """User percentage of the total CPU load."""
        return self.cpu["user_load"]

    @property
    def cpu_system_load(self) -> int:
        """System percentage of the total CPU load."""
        return self.cpu["system_load"]

    @property
    def cpu_total_load(self) -> int:
        """Total CPU load for Synology DSM."""
        system_load = self.cpu_system_load
        user_load = self.cpu_user_load
        other_load = self.cpu_other_load

        return system_load + user_load + other_load

    @property
    def cpu_1min_load(self) -> int:
        """Average CPU load past minute."""
        return self.cpu["1min_load"]

    @property
    def cpu_5min_load(self) -> int:
        """Average CPU load past 5 minutes."""
        return self.cpu["5min_load"]

    @property
    def cpu_15min_load(self) -> int:
        """Average CPU load past 15 minutes."""
        return self.cpu["15min_load"]

    @property
    def memory(self) -> MemoryUtilization:
        """Gets memory utilization."""
        return self._data["memory"]

    @property
    def memory_real_usage(self) -> int:
        """Real Memory usage from Synology DSM."""
        return self.memory["real_usage"]

    def memory_size(self, human_readable: bool = False) -> int | str:
        """Total memory size of Synology DSM."""
        return_data = self.memory["memory_size"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def memory_available_swap(self, human_readable: bool = False) -> int | str:
        """Total available memory swap."""
        # Memory is actually returned in KB's so multiply before converting
        return_data = self.memory["avail_swap"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def memory_cached(self, human_readable: bool = False) -> int | str:
        """Total cached memory."""
        # Memory is actually returned in KB's so multiply before converting
        return_data = self.memory["cached"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def memory_available_real(self, human_readable: bool = False) -> int | str:
        """Real available memory."""
        # Memory is actually returned in KB's so multiply before converting
        return_data = self.memory["avail_real"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def memory_total_real(self, human_readable: bool = False) -> int | str:
        """Total available real memory."""
        # Memory is actually returned in KB's so multiply before converting
        return_data = self.memory["total_real"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    def memory_total_swap(self, human_readable: bool = False) -> int | str:
        """Total swap memory."""
        # Memory is actually returned in KB's so multiply before converting
        return_data = self.memory["total_swap"] * 1024
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    @property
    def network(self) -> list[NetworkUtilization]:
        """Gets network utilization."""
        return self._data["network"]

    def _get_network(self, network_id: str) -> NetworkUtilization | None:
        """Function to get specific network (eth0, total, etc)."""
        for network in self.network:
            if network["device"] == network_id:
                return network
        return None

    def network_up(self, human_readable: bool = False) -> int | str | None:
        """Total upload speed being used."""
        if (network := self._get_network("total")) is not None:
            return_data = network["tx"]
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def network_down(self, human_readable: bool = False) -> int | str | None:
        """Total download speed being used."""
        if (network := self._get_network("total")) is not None:
            return_data = network["rx"]
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None



"""Helpers."""

from __future__ import annotations


class SynoFormatHelper:
    """Class containing various formatting functions."""

    @staticmethod
    def bytes_to_readable(num: int) -> str:
        """Converts bytes to a human readable format."""
        if num < 512:
            return "0 Kb"
        if num < 1024:
            return "1 Kb"

        work_num = float(num)
        for unit in ["", "Kb", "Mb", "Gb", "Tb", "Pb", "Eb", "Zb"]:
            if abs(work_num) < 1024.0:
                return f"{round(work_num, 1)}{unit}"
            work_num /= 1024.0
        return f"{round(work_num, 1)}Yb"

    @staticmethod
    def bytes_to_megabytes(num: int) -> float:
        """Converts bytes to megabytes."""
        var_mb = num / 1024.0 / 1024.0

        return round(var_mb, 1)

    @staticmethod
    def bytes_to_gigabytes(num: int) -> float:
        """Converts bytes to gigabytes."""
        var_gb = num / 1024.0 / 1024.0 / 1024.0

        return round(var_gb, 1)

    @staticmethod
    def bytes_to_terrabytes(num: int) -> float:
        """Converts bytes to terrabytes."""
        var_tb = num / 1024.0 / 1024.0 / 1024.0 / 1024.0

        return round(var_tb, 1)

    @staticmethod
    def megabytes_to_bytes(num: int | float) -> int:
        """Converts megabytes to bytes."""
        var_bytes = num * 1024.0 * 1024.0

        return int(round(var_bytes, 0))



"""DSM 5 SYNO.Storage.CGI.Storage data."""

from tests.const import UNIQUE_KEY

DSM_5_STORAGE_STORAGE_DS410J_RAID5_4DISKS_1VOL = {
    "disks": [
        {"id": "test_disk"},
        {
            "container": {
                "order": 0,
                "str": "DS410j",
                "supportPwrBtnDisable": False,
                "type": "internal",
            },
            "device": "/dev/sdd",
            "disable_secera": False,
            "diskType": "SATA",
            "erase_time": 374,
            "firm": "SC60",
            "has_system": True,
            "id": "sdd",
            "is4Kn": False,
            "isSsd": False,
            "is_erasing": False,
            "longName": "Disk 4",
            "model": "ST3000VN007-2E4166      ",
            "name": "Disk 4",
            "num_id": 4,
            "order": 4,
            "portType": "normal",
            "serial": "Z73095S2",
            "size_total": "3000592982016",
            "smart_status": "safe",
            "status": "normal",
            "support": False,
            "temp": 42,
            "used_by": "volume_1",
            "vendor": "Seagate",
        },
        {
            "container": {
                "order": 0,
                "str": "DS410j",
                "supportPwrBtnDisable": False,
                "type": "internal",
            },
            "device": "/dev/sdc",
            "disable_secera": False,
            "diskType": "SATA",
            "erase_time": 410,
            "firm": "80.00A80",
            "has_system": True,
            "id": "sdc",
            "is4Kn": False,
            "isSsd": False,
            "is_erasing": False,
            "longName": "Disk 3",
            "model": "WD30EZRZ-00Z5HB0        ",
            "name": "Disk 3",
            "num_id": 3,
            "order": 3,
            "portType": "normal",
            "serial": "WD-WCC4N0TEJ4F0",
            "size_total": "3000592982016",
            "smart_status": "safe",
            "status": "normal",
            "support": False,
            "temp": 42,
            "used_by": "volume_1",
            "vendor": "WDC     ",
        },
        {
            "container": {
                "order": 0,
                "str": "DS410j",
                "supportPwrBtnDisable": False,
                "type": "internal",
            },
            "device": "/dev/sdb",
            "disable_secera": False,
            "diskType": "SATA",
            "erase_time": 408,
            "firm": "82.00A82",
            "has_system": True,
            "id": "sdb",
            "is4Kn": False,
            "isSsd": False,
            "is_erasing": False,
            "longName": "Disk 2",
            "model": "WD30EFRX-68EUZN0        ",
            "name": "Disk 2",
            "num_id": 2,
            "order": 2,
            "portType": "normal",
            "serial": "WD-WCC4N6LSVCVX",
            "size_total": "3000592982016",
            "smart_status": "safe",
            "status": "normal",
            "support": False,
            "temp": 43,
            "used_by": "volume_1",
            "vendor": "WDC     ",
        },
        {
            "container": {
                "order": 0,
                "str": "DS410j",
                "supportPwrBtnDisable": False,
                "type": "internal",
            },
            "device": "/dev/sda",
            "disable_secera": False,
            "diskType": "SATA",
            "erase_time": 0,
            "firm": "82.00A82",
            "has_system": True,
            "id": "sda",
            "is4Kn": False,
            "isSsd": False,
            "is_erasing": False,
            "longName": "Disk 1",
            "model": "WD30EFRX-68N32N0        ",
            "name": "Disk 1",
            "num_id": 1,
            "order": 1,
            "portType": "normal",
            "serial": "WD-WCC7K5YA5H40",
            "size_total": "3000592982016",
            "smart_status": "90%",
            "status": "normal",
            "support": False,
            "temp": 44,
            "used_by": "volume_1",
            "vendor": "WDC     ",
        },
    ],
    "env": {
        "batchtask": {"max_task": 64, "remain_task": 64},
        "bay_number": "4",
        "ebox": [],
        "fs_acting": False,
        "is_space_actioning": False,
        "isns": {"address": "", "enabled": False},
        "isns_server": "",
        "max_fs_bytes": "17592181850112",
        "max_fs_bytes_high_end": "219902325555200",
        "model_name": "DS410j",
        "ram_enough_for_fs_high_end": False,
        "ram_size": 0,
        "ram_size_required": 32,
        "settingSwap": False,
        "showpooltab": False,
        "status": {"system_crashed": False, "system_need_repair": False},
        "support": {"ebox": False, "raid_cross": False, "sysdef": True},
        "unique_key": UNIQUE_KEY,
    },
    "hotSpares": [],
    "iscsiLuns": [
        {
            "can_do": {
                "data_scrubbing": True,
                "delete": True,
                "expand_by_disk": 1,
                "migrate": {"to_raid5+spare": "1-1", "to_raid6": 1},
            },
            "id": "iscsilun_LUN-1",
            "is_actioning": False,
            "iscsi_lun": {
                "blkNum": "19614744",
                "device_type": "file",
                "lid": 1,
                "location": "volume_1",
                "mapped_targets": [1],
                "name": "LUN-1",
                "restored_time": "0",
                "rootpath": "/volume1",
                "size": "10737418240",
                "thin_provision": False,
                "uuid": "fcf3a450-681c-06cb-fbb9-0400bdbe0780",
                "vaai_extent_size": "0",
                "vaai_support": False,
            },
            "num_id": 1,
            "progress": {"percent": "-1", "step": "none"},
            "status": "normal",
        }
    ],
    "iscsiTargets": [
        {
            "auth": {"mutual_username": "", "type": "none", "username": ""},
            "data_chksum": 0,
            "enabled": True,
            "hdr_chksum": 0,
            "iqn": "iqn.2000-01.com.synology:DiskStation.name",
            "mapped_logical_unit_number": [0],
            "mapped_luns": [1],
            "masking": [
                {"iqn": "iqn.2000-01.com.synology:default.acl", "permission": "rw"}
            ],
            "multi_sessions": False,
            "name": "Target-1",
            "num_id": 1,
            "recv_seg_bytes": 262144,
            "remote": [],
            "send_seg_bytes": 4096,
            "status": "online",
            "tid": 1,
        }
    ],
    "ports": [],
    "storagePools": [],
    "success": True,
    "volumes": [
        {"id": "test_volume"},
        {
            "can_do": {
                "data_scrubbing": True,
                "delete": True,
                "expand_by_disk": 1,
                "migrate": {"to_raid5+spare": "1-1", "to_raid6": 1},
            },
            "container": "internal",
            "device_type": "raid_5",
            "disk_failure_number": 0,
            "disks": ["sda", "sdb", "sdc", "sdd"],
            "eppool_used": "10042748928",
            "fs_type": "ext3",
            "id": "volume_1",
            "is_acting": False,
            "is_actioning": False,
            "is_inode_full": False,
            "is_writable": True,
            "max_fs_size": "17592181850112",
            "maximal_disk_size": "0",
            "minimal_disk_size": "2995767869440",
            "num_id": 1,
            "pool_path": "",
            "progress": {"percent": "-1", "step": "none"},
            "size": {
                "free_inode": "547237217",
                "total": "8846249701376",
                "total_device": "8987275100160",
                "total_inode": "548544512",
                "used": "5719795761152",
            },
            "space_path": "/dev/md2",
            "spares": [],
            "ssd_trim": {"support": "not support"},
            "status": "normal",
            "suggestions": [],
            "timebackup": True,
            "used_by_gluster": False,
            "vol_path": "/volume1",
            "vspace_can_do": {
                "drbd": {
                    "resize": {
                        "can_do": False,
                        "errCode": 53504,
                        "stopService": False,
                    }
                },
                "flashcache": {
                    "apply": {
                        "can_do": False,
                        "errCode": 53504,
                        "stopService": False,
                    },
                    "remove": {
                        "can_do": False,
                        "errCode": 53504,
                        "stopService": False,
                    },
                    "resize": {
                        "can_do": False,
                        "errCode": 53504,
                        "stopService": False,
                    },
                },
                "snapshot": {
                    "resize": {"can_do": True, "errCode": 0, "stopService": False}
                },
            },
        },
    ],
}



"""Test constants."""

# API test data are localized in
# `tests/api_data/dsm_[dsm_major_version]`
# Data constant names should be like :
# "DSM_[dsm_version]_[API_KEY]"
# if data failed, add "_FAILED"

SESSION_ID = "session_id"
SYNO_TOKEN = "Syñ0_T0k€ñ"  # noqa: S105
DEVICE_TOKEN = "Dév!cè_T0k€ñ"  # noqa: S105
UNIQUE_KEY = "1x2X3x!_UK"

# Common API error code
ERROR_UNKNOWN = {"error": {"code": 100}, "success": False}
ERROR_INVALID_PARAMETERS = {"error": {"code": 101}, "success": False}
ERROR_API_NOT_EXISTS = {"error": {"code": 102}, "success": False}
ERROR_API_METHOD_NOT_EXISTS = {"error": {"code": 103}, "success": False}
ERROR_API_VERSION_NOT_SUPPORTED = {"error": {"code": 104}, "success": False}
ERROR_INSUFFICIENT_USER_PRIVILEGE = {"error": {"code": 105}, "success": False}
ERROR_CONNECTION_TIME_OUT = {"error": {"code": 106}, "success": False}
ERROR_MULTIPLE_LOGIN_DETECTED = {"error": {"code": 107}, "success": False}

# Auth API error code
ERROR_AUTH_INVALID_CREDENTIALS = {"error": {"code": 400}, "success": False}
ERROR_AUTH_GUEST_OR_DISABLED_ACCOUNT = {"error": {"code": 401}, "success": False}
ERROR_AUTH_PERMISSION_DENIED = {"error": {"code": 402}, "success": False}
ERROR_AUTH_OTP_NOT_SPECIFIED = {"error": {"code": 403}, "success": False}
ERROR_AUTH_OTP_AUTHENTICATE_FAILED = {"error": {"code": 404}, "success": False}
ERROR_AUTH_INCORRECT_APP_PORTAL = {"error": {"code": 405}, "success": False}
ERROR_AUTH_OTP_CODE_ENFORCED = {"error": {"code": 406}, "success": False}
ERROR_AUTH_MAX_TRIES = {"error": {"code": 407}, "success": False}



"""Library exceptions."""

from __future__ import annotations

from .const import (
    API_AUTH,
    ERROR_AUTH,
    ERROR_COMMON,
    ERROR_DOWNLOAD_SEARCH,
    ERROR_DOWNLOAD_TASK,
    ERROR_FILE,
    ERROR_SURVEILLANCE,
    ERROR_VIRTUALIZATION,
)


class SynologyDSMException(Exception):
    """Generic Synology DSM exception."""

    def __init__(self, api: str | None, code: int, details: str | None = None) -> None:
        """Constructor method."""
        reason = ERROR_COMMON.get(code)
        if api and not reason:
            if api == API_AUTH:
                reason = ERROR_AUTH.get(code)
            elif "SYNO.DownloadStation" in api:
                if "BTSearch" in api:
                    reason = ERROR_DOWNLOAD_SEARCH.get(code)
                elif "Task" in api:
                    reason = ERROR_DOWNLOAD_TASK.get(code)
            elif "SYNO.FileStation" in api:
                reason = ERROR_FILE.get(code)
            elif "SYNO.SurveillanceStation" in api:
                reason = ERROR_SURVEILLANCE.get(code)
            elif "SYNO.Virtualization" in api:
                reason = ERROR_VIRTUALIZATION.get(code)
        if not reason:
            reason = "Unknown"

        error_message = {"api": api, "code": code, "reason": reason, "details": details}
        super().__init__(error_message)


class SynologyDSMNotLoggedInException(SynologyDSMException):
    """Not logged in exception."""

    def __init__(self) -> None:
        """Constructor method."""
        super().__init__(None, -1, "Not logged in. You have to do login() first.")


# Request
class SynologyDSMRequestException(SynologyDSMException):
    """Request exception."""

    def __init__(self, exception: Exception) -> None:
        """Constructor method."""
        ex_class = exception.__class__.__name__
        if not exception.args:
            super().__init__(None, -1, ex_class)
            return
        ex_reason = exception.args[0]
        if hasattr(exception.args[0], "reason"):
            ex_reason = exception.args[0].reason
        super().__init__(None, -1, f"{ex_class} = {ex_reason}")


# API
class SynologyDSMAPINotExistsException(SynologyDSMException):
    """API not exists exception."""

    def __init__(self, api: str) -> None:
        """Constructor method."""
        super().__init__(api, -2, f"API {api} does not exists")


class SynologyDSMAPIErrorException(SynologyDSMException):
    """API returns an error exception."""

    def __init__(self, api: str, code: int, details: str) -> None:
        """Constructor method."""
        super().__init__(api, code, details)


# Login
class SynologyDSMLoginFailedException(SynologyDSMException):
    """Failed to login exception."""

    def __init__(self, code: int, details: str | None = None) -> None:
        """Constructor method."""
        super().__init__(API_AUTH, code, details)


class SynologyDSMLoginInvalidException(SynologyDSMLoginFailedException):
    """Invalid password & not admin account exception."""

    def __init__(self, username: str) -> None:
        """Constructor method."""
        message = f"Invalid password or not admin account: {username}"
        super().__init__(400, message)


class SynologyDSMLoginDisabledAccountException(SynologyDSMLoginFailedException):
    """Guest & disabled account exception."""

    def __init__(self, username: str) -> None:
        """Constructor method."""
        message = f"Guest or disabled account: {username}"
        super().__init__(401, message)


class SynologyDSMLoginPermissionDeniedException(SynologyDSMLoginFailedException):
    """No access to login exception."""

    def __init__(self, username: str) -> None:
        """Constructor method."""
        message = f"Permission denied for account: {username}"
        super().__init__(402, message)


class SynologyDSMLogin2SARequiredException(SynologyDSMLoginFailedException):
    """2SA required to login exception."""

    def __init__(self, username: str) -> None:
        """Constructor method."""
        message = f"Two-step authentication required for account: {username}"
        super().__init__(403, message)


class SynologyDSMLogin2SAFailedException(SynologyDSMLoginFailedException):
    """2SA code failed exception."""

    def __init__(self) -> None:
        """Constructor method."""
        message = "Two-step authentication failed, retry with a new pass code"
        super().__init__(404, message)


class SynologyDSMLogin2SAForcedException(SynologyDSMLoginFailedException):
    """2SA force to setup exception."""

    def __init__(self, username: str) -> None:
        """Constructor method."""
        message = (
            f"Two-step authentication forced to be setuped for account: {username}"
        )
        super().__init__(406, message)



"""DSM 6 SYNO.Core.Share data."""

DSM_6_CORE_SHARE = {
    "data": {
        "shares": [
            {
                "desc": "Docker Containers",
                "enable_recycle_bin": False,
                "enable_share_compress": False,
                "enable_share_cow": True,
                "enc_auto_mount": False,
                "encryption": 0,
                "force_readonly_reason": "",
                "hidden": True,
                "is_aclmode": True,
                "is_block_snap_action": False,
                "is_cluster_share": False,
                "is_cold_storage_share": False,
                "is_exfat_share": False,
                "is_force_readonly": False,
                "is_share_moving": False,
                "is_support_acl": True,
                "is_sync_share": False,
                "is_usb_share": False,
                "name": "docker",
                "quota_value": 0,
                "recycle_bin_admin_only": False,
                "share_quota_used": 0,
                "support_action": 511,
                "support_snapshot": True,
                "task_id": "",
                "unite_permission": False,
                "uuid": "78egut02-b5b1-4933-adt8-a9208526d234",
                "vol_path": "/volume1",
            },
            {
                "desc": "",
                "enable_recycle_bin": True,
                "enable_share_compress": False,
                "enable_share_cow": True,
                "enc_auto_mount": False,
                "encryption": 0,
                "force_readonly_reason": "",
                "hidden": False,
                "is_aclmode": True,
                "is_block_snap_action": False,
                "is_cluster_share": False,
                "is_cold_storage_share": False,
                "is_exfat_share": False,
                "is_force_readonly": False,
                "is_share_moving": False,
                "is_support_acl": True,
                "is_sync_share": False,
                "is_usb_share": False,
                "name": "test_share",
                "quota_value": 0,
                "recycle_bin_admin_only": False,
                "share_quota_used": 36146658672640.0,
                "support_action": 511,
                "support_snapshot": True,
                "task_id": "",
                "unite_permission": False,
                "uuid": "2ee6c06a-8766-48b5-013d-63b18652a393",
                "vol_path": "/volume1",
            },
            {
                "desc": "user home",
                "enable_recycle_bin": False,
                "enable_share_compress": False,
                "enable_share_cow": True,
                "enc_auto_mount": False,
                "encryption": 0,
                "force_readonly_reason": "",
                "hidden": False,
                "is_aclmode": True,
                "is_block_snap_action": False,
                "is_cluster_share": False,
                "is_cold_storage_share": False,
                "is_exfat_share": False,
                "is_force_readonly": False,
                "is_share_moving": False,
                "is_support_acl": True,
                "is_sync_share": False,
                "is_usb_share": False,
                "name": "homes",
                "quota_value": 0,
                "recycle_bin_admin_only": False,
                "share_quota_used": 0.015625,
                "support_action": 511,
                "support_snapshot": True,
                "task_id": "",
                "unite_permission": False,
                "uuid": "2b829t90-9512-4236-qqe0-d4133e9992d0",
                "vol_path": "/volume1",
            },
            {
                "desc": "Log volume",
                "enable_recycle_bin": True,
                "enable_share_compress": True,
                "enable_share_cow": True,
                "enc_auto_mount": True,
                "encryption": 0,
                "force_readonly_reason": "",
                "hidden": True,
                "is_aclmode": True,
                "is_block_snap_action": False,
                "is_cluster_share": False,
                "is_cold_storage_share": False,
                "is_exfat_share": False,
                "is_force_readonly": False,
                "is_share_moving": False,
                "is_support_acl": True,
                "is_sync_share": False,
                "is_usb_share": False,
                "name": "logs",
                "quota_value": 0,
                "recycle_bin_admin_only": True,
                "share_quota_used": 947.28515625,
                "support_action": 511,
                "support_snapshot": True,
                "task_id": "",
                "unite_permission": False,
                "uuid": "b9876507-6880-4wes-8d61-6c984c0813ty",
                "vol_path": "/volume2",
            },
            {
                "desc": "VMs",
                "enable_recycle_bin": False,
                "enable_share_compress": False,
                "enable_share_cow": True,
                "enc_auto_mount": False,
                "encryption": 0,
                "force_readonly_reason": "",
                "hidden": False,
                "is_aclmode": True,
                "is_block_snap_action": False,
                "is_cluster_share": False,
                "is_cold_storage_share": False,
                "is_exfat_share": False,
                "is_force_readonly": False,
                "is_share_moving": False,
                "is_support_acl": True,
                "is_sync_share": False,
                "is_usb_share": False,
                "name": "Virtual_Machines",
                "quota_value": 0,
                "recycle_bin_admin_only": False,
                "share_quota_used": 33911668,
                "support_action": 511,
                "support_snapshot": True,
                "task_id": "",
                "unite_permission": False,
                "uuid": "5416f693-04tt-4re2-b8e4-f6b18731689b",
                "vol_path": "/volume3",
            },
        ],
        "total": 5,
    },
    "success": True,
}



"""DSM external USB device data."""

from __future__ import annotations

from typing import TypedDict, cast

from synology_dsm.api import SynoBaseApi
from synology_dsm.helpers import SynoFormatHelper

ExternalUsbDevicePartitionDataType = TypedDict(
    "ExternalUsbDevicePartitionDataType",
    {
        "dev_fstype": str,
        "filesystem": str,
        "name_id": str,
        "partition_title": str,
        "share_name": str,
        "status": str,
        "total_size_mb": "int | str",
        "used_size_mb": "int | None",
    },
    total=False,
)
ExternalUsbDeviceDataType = TypedDict(
    "ExternalUsbDeviceDataType",
    {
        "dev_id": str,
        "dev_title": str,
        "dev_type": str,
        "formatable": bool,
        "partitions": "dict[str, SynoUSBStoragePartition]",
        "producer": str,
        "product": "str | None",
        "progress": str,
        "status": str,
        "total_size_mb": int,
    },
    total=False,
)


class SynoCoreExternalUSB(SynoBaseApi["dict[str, SynoCoreExternalUSBDevice]"]):
    """Class for external USB storage devices."""

    API_KEY = "SYNO.Core.ExternalDevice.Storage.USB"
    REQUEST_DATA = {"additional": '["all"]'}

    async def update(self) -> None:
        """Updates external USB storage device data."""
        self._data: dict[str, SynoCoreExternalUSBDevice] = {}
        raw_data = await self._dsm.post(self.API_KEY, "list", data=self.REQUEST_DATA)
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            for device in data["devices"]:
                self._data[device["dev_id"]] = SynoCoreExternalUSBDevice(device)

    # Root
    @property
    def get_devices(self) -> dict[str, SynoCoreExternalUSBDevice]:
        """Gets all external USB storage devices."""
        return self._data

    def get_device(self, device_id: str) -> SynoCoreExternalUSBDevice | None:
        """Returns a specific external USB storage device."""
        return self._data.get(device_id)


class SynoCoreExternalUSBDevice:
    """A representation of an external USB device."""

    def __init__(self, data: dict):
        """Initialize a external USB device."""
        partitions: dict[str, SynoUSBStoragePartition] = {}
        for partition in data["partitions"]:
            partitions[partition["name_id"]] = SynoUSBStoragePartition(partition)
        self._data = cast(ExternalUsbDeviceDataType, {**data, "partitions": partitions})

    @property
    def device_id(self) -> str:
        """Return id of the device."""
        return self._data["dev_id"]

    @property
    def device_name(self) -> str:
        """The title of the external USB storage device."""
        return self._data["dev_title"]

    @property
    def device_type(self) -> str:
        """The type of the external USB storage device."""
        return self._data["dev_type"]

    def device_size_total(self, human_readable: bool = False) -> str | int:
        """Total size of the external USB storage device."""
        return_data = SynoFormatHelper.megabytes_to_bytes(
            int(self._data["total_size_mb"])
        )
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(return_data)
        return return_data

    @property
    def device_status(self) -> str:
        """The status of the external USB storage device."""
        return self._data["status"]

    @property
    def device_formatable(self) -> bool:
        """Whether the external USB storage device can be formatted."""
        return self._data["formatable"]

    @property
    def device_progress(self) -> str:
        """The progress the external USB storage device."""
        return self._data["progress"]

    @property
    def device_product_name(self) -> str | None:
        """The product name of the external USB storage device."""
        return self._data.get("product")

    @property
    def device_manufacturer(self) -> str:
        """The producer name of the external USB storage device."""
        return self._data["producer"]

    # Partition
    @property
    def device_partitions(self) -> dict[str, SynoUSBStoragePartition]:
        """Returns all partitions of the external USB storage device."""
        return self._data["partitions"]

    def get_device_partition(self, partition_id: str) -> SynoUSBStoragePartition | None:
        """Returns a partition of the external USB storage device."""
        return self._data["partitions"].get(partition_id)

    def partitions_all_size_total(
        self, human_readable: bool = False
    ) -> str | int | None:
        """Total size of all parititions of the external USB storage device."""
        partitions = self._data["partitions"]
        if not partitions:
            return None

        size_total = 0
        for partition in partitions.values():
            partition_size = partition.partition_size_total()
            # Partitions may be reported without a size
            if isinstance(partition_size, int):
                size_total += partition_size

        if human_readable:
            return SynoFormatHelper.bytes_to_readable(size_total)
        return size_total

    def partitions_all_size_used(
        self, human_readable: bool = False
    ) -> str | int | None:
        """Total size used of all partitions of the external USB storage device."""
        partitions = self._data["partitions"]
        if not partitions:
            return None

        size_used = 0
        for partition in partitions.values():
            partition_used = partition.partition_size_used()
            # Partitions may be reported without a size
            if isinstance(partition_used, int):
                size_used += partition_used

        if human_readable:
            return SynoFormatHelper.bytes_to_readable(size_used)
        return size_used

    @property
    def partitions_all_percentage_used(self) -> float | None:
        """Used size in percentage for all partitions of the USB storage device."""
        size_total = self.partitions_all_size_total()
        size_used = self.partitions_all_size_used()

        if (
            isinstance(size_used, int)
            and size_used >= 0
            and isinstance(size_total, int)
            and size_total > 0
        ):
            return round((float(size_used) / float(size_total)) * 100.0, 1)
        return None


class SynoUSBStoragePartition:
    """A representation of a parition of an external USB storage device."""

    def __init__(self, data: ExternalUsbDevicePartitionDataType):
        """Initialize a partition object of an external USB storage device."""
        self._data = data

    @property
    def fstype(self) -> str:
        """Return the dev_fstype for the partition."""
        return self._data["dev_fstype"]

    @property
    def filesystem(self) -> str:
        """Return the filesystem for the partition."""
        return self._data["filesystem"]

    @property
    def name_id(self) -> str:
        """Return the name_id for the partition."""
        return self._data["name_id"]

    @property
    def partition_title(self) -> str:
        """Return the title for the partition."""
        return self._data["partition_title"]

    @property
    def share_name(self) -> str:
        """Return the share name for the partition."""
        return self._data["share_name"]

    @property
    def status(self) -> str:
        """Return the status for the partition."""
        return self._data["status"]

    def partition_size_total(self, human_readable: bool = False) -> int | str | None:
        """Total size of the partition."""
        # API returns property as empty string if a partition has no size
        size_total = self._data["total_size_mb"]
        if not isinstance(size_total, int):
            return None
        size_total = SynoFormatHelper.megabytes_to_bytes(size_total)
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(size_total)
        return size_total

    def partition_size_used(self, human_readable: bool = False) -> int | str | None:
        """Used size of the partition."""
        # API does not return property if a partition has no size
        size_used = self._data.get("used_size_mb")
        if not isinstance(size_used, int):
            return None
        size_used = SynoFormatHelper.megabytes_to_bytes(size_used)
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(size_used)
        return size_used

    @property
    def partition_percentage_used(self) -> float | None:
        """Used size in percentage of the partition."""
        size_total = self.partition_size_total()
        size_used = self.partition_size_used()
        if (
            isinstance(size_used, int)
            and size_used >= 0
            and isinstance(size_total, int)
            and size_total > 0
        ):
            return round((float(size_used) / float(size_total)) * 100.0, 1)
        return None

    @property
    def is_mounted(self) -> bool:
        """Is the partition formatted."""
        return self._data["share_name"] != ""

    @property
    def is_supported(self) -> bool:
        """Is the partition formatted."""
        return self._data["filesystem"] != ""



"""DSM 7 SYNO.Core.Upgrade data."""

DSM_7_CORE_UPGRADE_FALSE = {"data": {"update": {"available": False}}, "success": True}
DSM_7_CORE_UPGRADE_TRUE = {
    "data": {
        "update": {
            "available": True,
            "reboot": "now",
            "restart": "some",
            "type": "nano",
            "version": "7.0.1-42218 Update 3",
            "version_details": {
                "buildnumber": 42218,
                "major": 7,
                "micro": 1,
                "minor": 0,
                "nano": 3,
                "os_name": "DSM",
            },
        }
    },
    "success": True,
}



"""DSM 5 datas."""

from .const_5_api_auth import (
    DSM_5_AUTH_LOGIN,
    DSM_5_AUTH_LOGIN_2SA,
    DSM_5_AUTH_LOGIN_2SA_OTP,
)
from .const_5_api_info import DSM_5_API_INFO
from .core.const_5_core_utilization import DSM_5_CORE_UTILIZATION
from .dsm.const_5_dsm_info import DSM_5_DSM_INFORMATION
from .dsm.const_5_dsm_network import DSM_5_DSM_NETWORK
from .storage.const_5_storage_storage import (
    DSM_5_STORAGE_STORAGE_DS410J_RAID5_4DISKS_1VOL,
)

__all__ = [
    "DSM_5_AUTH_LOGIN",
    "DSM_5_AUTH_LOGIN_2SA",
    "DSM_5_AUTH_LOGIN_2SA_OTP",
    "DSM_5_API_INFO",
    "DSM_5_CORE_UTILIZATION",
    "DSM_5_DSM_INFORMATION",
    "DSM_5_DSM_NETWORK",
    "DSM_5_STORAGE_STORAGE_DS410J_RAID5_4DISKS_1VOL",
]



"""Synology API models."""

from __future__ import annotations

from typing import TYPE_CHECKING, Generic, TypeVar

if TYPE_CHECKING:
    from synology_dsm import SynologyDSM


_DataT = TypeVar("_DataT")


class SynoBaseApi(Generic[_DataT]):
    """Base api class."""

    def __init__(self, dsm: "SynologyDSM") -> None:
        """Constructor method."""
        self._dsm = dsm
        self._data: _DataT = {}  # type: ignore[assignment]



"""Shared Folders data."""

from __future__ import annotations

from typing import TypedDict

from synology_dsm.api import SynoBaseApi
from synology_dsm.helpers import SynoFormatHelper

Share = TypedDict(
    "Share",
    {
        "uuid": str,
        "name": str,
        "vol_path": str,
        "enable_recycle_bin": bool,
        "share_quota_used": float,
    },
    total=False,
)


class ShareDataType(TypedDict):
    """Data type."""

    shares: list[Share]


class SynoCoreShare(SynoBaseApi[ShareDataType]):
    """Class containing Share data."""

    API_KEY = "SYNO.Core.Share"
    # Syno supports two methods to retrieve resource details, GET and POST.
    # GET returns a limited set of keys. With POST the same keys as GET
    # are returned plus any keys listed in the "additional" parameter.
    # NOTE: The value of the additional key must be a string.
    REQUEST_DATA = {
        "additional": '["hidden","encryption","is_aclmode","unite_permission",'
        '"is_support_acl","is_sync_share","is_force_readonly","force_readonly_reason",'
        '"recyclebin","is_share_moving","is_cluster_share","is_exfat_share",'
        '"is_cold_storage_share","support_snapshot","share_quota",'
        '"enable_share_compress","enable_share_cow","include_cold_storage_share",'
        '"is_cold_storage_share"]',
        "shareType": "all",
    }

    async def update(self) -> None:
        """Updates share data."""
        raw_data = await self._dsm.post(self.API_KEY, "list", data=self.REQUEST_DATA)
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data

    @property
    def shares(self) -> list[Share]:
        """Gets all shares."""
        return self._data["shares"]

    @property
    def shares_uuids(self) -> list[str]:
        """Return (internal) share ids."""
        shares = []
        for share in self.shares:
            shares.append(share["uuid"])
        return shares

    def get_share(self, share_uuid: str) -> Share:
        """Returns a specific share by uuid.."""
        for share in self.shares:
            if share["uuid"] == share_uuid:
                return share
        return {}

    def share_name(self, share_uuid: str) -> str:
        """Return the name of this share."""
        return self.get_share(share_uuid)["name"]

    def share_path(self, share_uuid: str) -> str:
        """Return the volume path of this share."""
        return self.get_share(share_uuid)["vol_path"]

    def share_recycle_bin(self, share_uuid: str) -> bool:
        """Is the recycle bin enabled for this share?"""
        return self.get_share(share_uuid)["enable_recycle_bin"]

    def share_size(self, share_uuid: str, human_readable: bool = False) -> int | str:
        """Total size of share."""
        share_size_mb = self.get_share(share_uuid)["share_quota_used"]
        # Share size is returned in MB so we convert it.
        share_size_bytes = SynoFormatHelper.megabytes_to_bytes(share_size_mb)
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(share_size_bytes)
        return share_size_bytes



"""DSM 6 SYNO.Core.SecurityScan.Status data."""

DSM_6_CORE_SECURITY = {
    "data": {
        "items": {
            "malware": {
                "category": "malware",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 3,
                "waitNum": 0,
            },
            "network": {
                "category": "network",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 4,
                "waitNum": 0,
            },
            "securitySetting": {
                "category": "securitySetting",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 0,
                "waitNum": 0,
            },
            "systemCheck": {
                "category": "systemCheck",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 6,
                "waitNum": 0,
            },
            "update": {
                "category": "update",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 4,
                "waitNum": 0,
            },
            "userInfo": {
                "category": "userInfo",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 6,
                "waitNum": 0,
            },
        },
        "lastScanTime": "1588298442",
        "startTime": "",
        "success": True,
        "sysProgress": 100,
        "sysStatus": "safe",
    },
    "success": True,
}

DSM_6_CORE_SECURITY_UPDATE_OUTOFDATE = {
    "data": {
        "items": {
            "malware": {
                "category": "malware",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 3,
                "waitNum": 0,
            },
            "network": {
                "category": "network",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 4,
                "waitNum": 0,
            },
            "securitySetting": {
                "category": "securitySetting",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 0,
                "waitNum": 0,
            },
            "systemCheck": {
                "category": "systemCheck",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 6,
                "waitNum": 0,
            },
            "update": {
                "category": "update",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 1,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "outOfDate",
                "progress": 100,
                "runningItem": "",
                "total": 4,
                "waitNum": 0,
            },
            "userInfo": {
                "category": "userInfo",
                "fail": {
                    "danger": 0,
                    "info": 0,
                    "outOfDate": 0,
                    "risk": 0,
                    "warning": 0,
                },
                "failSeverity": "safe",
                "progress": 100,
                "runningItem": "",
                "total": 6,
                "waitNum": 0,
            },
        },
        "lastScanTime": "1590717640",
        "startTime": "",
        "success": True,
        "sysProgress": 100,
        "sysStatus": "outOfDate",
    },
    "success": True,
}



"""DSM 7 SYNO.DSM.Info data."""

DSM_7_DSM_INFORMATION_DS213_PLUS = {
    "data": {
        "codepage": "enu",
        "model": "DS213+",
        "ram": 512,
        "serial": "XXXXXXXXXXX",
        "temperature": 30,
        "temperature_warn": False,
        "time": "Thu Apr 30 14:57:35 2020",
        "uptime": 3258607,
        "version": "24922",
        "version_string": "DSM 7.0-41222",
    },
    "success": True,
}

DSM_7_DSM_INFORMATION_DS918_PLUS = {
    "data": {
        "codepage": "fre",
        "model": "DS918+",
        "ram": 4096,
        "serial": "1920PDN001501",
        "temperature": 40,
        "temperature_warn": False,
        "time": "Sun Mar 29 19:33:41 2020",
        "uptime": 155084,
        "version": "24922",
        "version_string": "DSM 7.0-41222",
    },
    "success": True,
}

DSM_7_DSM_INFORMATION = DSM_7_DSM_INFORMATION_DS918_PLUS



"""Synology FileStation API wrapper."""

from __future__ import annotations

from collections.abc import AsyncIterator
from io import BufferedReader

import aiofiles
from aiohttp import StreamReader

from synology_dsm.api import SynoBaseApi

from .models import (
    SynoFileAdditionalOwner,
    SynoFileFile,
    SynoFileFileAdditional,
    SynoFileFileAdditionalPermission,
    SynoFileFileAdditionalTime,
    SynoFileSharedFolder,
    SynoFileSharedFolderAdditional,
    SynoFileSharedFolderAdditionalPermission,
    SynoFileSharedFolderAdditionalVolumeStatus,
)


class SynoFileStation(SynoBaseApi):
    """An implementation of a Synology FileStation."""

    API_KEY = "SYNO.FileStation.*"
    LIST_API_KEY = "SYNO.FileStation.List"
    DOWNLOAD_API_KEY = "SYNO.FileStation.Download"
    UPLOAD_API_KEY = "SYNO.FileStation.Upload"
    DELETE_API_KEY = "SYNO.FileStation.Delete"

    async def get_shared_folders(
        self, offset: int = 0, limit: int = 100, only_writable: bool = False
    ) -> list[SynoFileSharedFolder] | None:
        """Get a list of all shared folders."""
        raw_data = await self._dsm.get(
            self.LIST_API_KEY,
            "list_share",
            {
                "offset": offset,
                "limit": limit,
                "onlywritable": only_writable,
                "additional": (
                    '["real_path","owner","time","perm",'
                    '"mount_point_type","sync_share","volume_status"]'
                ),
            },
        )
        if not isinstance(raw_data, dict) or (data := raw_data.get("data")) is None:
            return None

        shared_folders: list[SynoFileSharedFolder] = []
        for folder in data["shares"]:
            if (additional := folder.get("additional")) is not None:
                shared_folders.append(
                    SynoFileSharedFolder(
                        SynoFileSharedFolderAdditional(
                            additional["mount_point_type"],
                            SynoFileAdditionalOwner(**additional["owner"]),
                            SynoFileSharedFolderAdditionalPermission(
                                **additional["perm"]
                            ),
                            SynoFileSharedFolderAdditionalVolumeStatus(
                                **additional["volume_status"],
                            ),
                        ),
                        folder["isdir"],
                        folder["name"],
                        folder["path"],
                    )
                )
            else:
                shared_folders.append(
                    SynoFileSharedFolder(
                        None, folder["isdir"], folder["name"], folder["path"]
                    )
                )

        return shared_folders

    async def get_files(
        self, path: str, offset: int = 0, limit: int = 100
    ) -> list[SynoFileFile] | None:
        """Get a list of all files in a folder."""
        raw_data = await self._dsm.get(
            self.LIST_API_KEY,
            "list",
            {
                "offset": offset,
                "limit": limit,
                "folder_path": path,
                "additional": (
                    '["real_path","owner","time","perm",'
                    '"mount_point_type","type","size"]'
                ),
            },
        )
        if not isinstance(raw_data, dict) or (data := raw_data.get("data")) is None:
            return None

        files: list[SynoFileFile] = []
        for file in data["files"]:
            if (additional := file.get("additional")) is not None:
                files.append(
                    SynoFileFile(
                        SynoFileFileAdditional(
                            additional["mount_point_type"],
                            SynoFileAdditionalOwner(**additional["owner"]),
                            SynoFileFileAdditionalPermission(**additional["perm"]),
                            additional["real_path"],
                            additional["size"],
                            SynoFileFileAdditionalTime(**additional["time"]),
                            additional["type"],
                        ),
                        file["isdir"],
                        file["name"],
                        file["path"],
                    )
                )
            else:
                files.append(
                    SynoFileFile(None, file["isdir"], file["name"], file["path"])
                )

        return files

    async def upload_file(
        self,
        path: str,
        filename: str,
        source: bytes | BufferedReader | AsyncIterator[bytes] | str,
        create_parents: bool = False,
    ) -> bool | None:
        """Upload a file to a folder from eather a local source_file or content."""
        if isinstance(source, str):
            source = open(source, "rb")

        raw_data = await self._dsm.post(
            self.UPLOAD_API_KEY,
            "upload",
            path=path,
            filename=filename,
            content=source,
            create_parents=create_parents,
        )
        if not isinstance(raw_data, dict):
            return None
        return raw_data.get("success")

    async def download_file(
        self, path: str, filename: str, target_file: str | None = None
    ) -> StreamReader | bool | None:
        """Download a file to local target_file or returns an async StreamReader."""
        response_content = await self._dsm.get(
            self.DOWNLOAD_API_KEY,
            "download",
            {"path": f"{path}/{filename}", "mode": "download"},
            raw_response_content=True,
        )
        if not isinstance(response_content, StreamReader):
            return None

        if target_file:
            async with aiofiles.open(target_file, "wb") as fh:
                async for data, _ in response_content.iter_chunks():
                    await fh.write(data)
            return True

        return response_content

    async def delete_file(self, path: str, filename: str) -> bool | None:
        """Delete a file."""
        raw_data = await self._dsm.get(
            self.DELETE_API_KEY,
            "delete",
            {"path": f"{path}/{filename}", "recursive": False},
        )
        if not isinstance(raw_data, dict):
            return None
        return raw_data.get("success")



"""Synology SurveillanceStation API wrapper."""

from __future__ import annotations

from typing import TypedDict, cast

from synology_dsm.api import SynoBaseApi

from .camera import SynoCamera, SynoCameraLiveView
from .const import (
    MOTION_DETECTION_BY_SURVEILLANCE,
    MOTION_DETECTION_DISABLED,
    SNAPSHOT_PROFILE_BALANCED,
)

SurveillanceStationInfoData = TypedDict(
    "SurveillanceStationInfoData",
    {
        "CMSMinVersion": str,
        "DSModelName": str,
        "SvsClientMinVersion": str,
        "VS360HDLoginMinVersion": str,
        "VS960HDMinVersion": str,
        "cameraNumber": int,
        "defaultWallpaperCount": int,
        "enableVideoRelay": bool,
        "hostname": str,
        "inaAdvancedPriv": int,
        "isBeta": bool,
        "isLicenseEnough": int,
        "is_beta": bool,
        "liscenseNumber": int,
        "maxCameraSupport": int,
        "maxlanport": str,
        "path": str,
        "pluginHelperVersion": str,
        "productName": str,
        "remindQuickconnectTunnel": bool,
        "reportURL": str,
        "serial": str,
        "serviceVolSize": float,
        "strInaAdvancedPriv": str,
        "timezone": str,
        "timezoneTZDB": str,
        "uid": int,
        "unique": str,
        "userPriv": int,
        "version": dict,
        "webPluginVersion": str,
    },
    total=False,
)

SurveillanceStationInfo = TypedDict(
    "SurveillanceStationInfo", {"data": SurveillanceStationInfoData, "success": bool}
)

SurveillanceStationTakeSnapshotDataType = TypedDict(
    "SurveillanceStationTakeSnapshotDataType", {"id": int}
)

SurveillanceStationTakeSnapshotType = TypedDict(
    "SurveillanceStationTakeSnapshotType",
    {"data": SurveillanceStationTakeSnapshotDataType, "success": bool},
)


class SynoSurveillanceStation(SynoBaseApi["dict[int, SynoCamera]"]):
    """An implementation of a Synology SurveillanceStation."""

    API_KEY = "SYNO.SurveillanceStation.*"
    INFO_API_KEY = "SYNO.SurveillanceStation.Info"
    CAMERA_API_KEY = "SYNO.SurveillanceStation.Camera"
    CAMERA_EVENT_API_KEY = "SYNO.SurveillanceStation.Camera.Event"
    HOME_MODE_API_KEY = "SYNO.SurveillanceStation.HomeMode"
    SNAPSHOT_API_KEY = "SYNO.SurveillanceStation.SnapShot"

    async def update(self) -> None:
        """Update cameras and motion settings with latest from API."""
        self._data: dict[int, SynoCamera] = {}
        raw_data = await self._dsm.get(self.CAMERA_API_KEY, "List", max_version=7)
        if not isinstance(raw_data, dict) or (data := raw_data.get("data")) is None:
            return

        for camera_data in data["cameras"]:
            if camera_data["id"] in self._data:
                self._data[camera_data["id"]].update(camera_data)
            else:
                self._data[camera_data["id"]] = SynoCamera(camera_data)

        for camera_id, camera in self._data.items():
            motion_raw_data = await self._dsm.get(
                self.CAMERA_EVENT_API_KEY, "MotionEnum", {"camId": camera_id}
            )
            if (
                isinstance(motion_raw_data, dict)
                and (motion_data := motion_raw_data.get("data")) is not None
            ):
                camera.update_motion_detection(motion_data)

        if not self._data:
            return

        live_view_raw_datas = await self._dsm.get(
            self.CAMERA_API_KEY,
            "GetLiveViewPath",
            {"idList": ",".join(str(k) for k in self._data)},
        )
        if (
            isinstance(live_view_raw_datas, dict)
            and (live_view_datas := live_view_raw_datas.get("data")) is not None
        ):
            for live_view_data in live_view_datas:
                self._data[live_view_data["id"]].live_view.update(live_view_data)

    # Global
    async def get_info(self) -> SurveillanceStationInfo | None:
        """Return general informations about the Surveillance Station instance."""
        raw_data = await self._dsm.get(self.INFO_API_KEY, "GetInfo")
        if isinstance(raw_data, dict) and raw_data.get("success"):
            return cast(SurveillanceStationInfo, raw_data)
        return None

    # Camera
    def get_all_cameras(self) -> list[SynoCamera]:
        """Return a list of cameras."""
        return list(self._data.values())

    def get_camera(self, camera_id: int) -> SynoCamera | None:
        """Return camera matching camera_id."""
        return self._data.get(camera_id)

    def get_camera_live_view_path(
        self, camera_id: int, video_format: str | None = None
    ) -> SynoCameraLiveView | str | None:
        """Return camera live view path matching camera_id.

        Args:
            camera_id: ID of the camera we want to get the live view path.
            video_format: mjpeg_http | multicast | mxpeg_http |  rtsp_http | rtsp.
        """
        if (camera := self._data.get(camera_id)) is None:
            return None
        if video_format:
            return getattr(camera.live_view, video_format, None)
        return camera.live_view

    async def get_camera_image(
        self, camera_id: int, profile: int = SNAPSHOT_PROFILE_BALANCED
    ) -> bytes:
        """Return bytes of camera image for camera matching camera_id.

        Args:
            camera_id: ID of the camera we want to take a snapshot from
            profile: SNAPSHOT_PROFILE_HIGH_QUALITY |
                     SNAPSHOT_PROFILE_BALANCED |
                     SNAPSHOT_PROFILE_LOW_BANDWIDTH
        """
        return cast(
            bytes,
            await self._dsm.get(
                self.CAMERA_API_KEY,
                "GetSnapshot",
                {"id": camera_id, "cameraId": camera_id, "profileType": profile},
            ),
        )

    async def enable_camera(self, camera_ids: str) -> bool | None:
        """Enable camera(s) - multiple ID or single ex 1 or 1,2,3."""
        raw_data = await self._dsm.get(
            self.CAMERA_API_KEY, "Enable", {"idList": camera_ids}
        )
        if (
            isinstance(raw_data, dict)
            and (result := raw_data.get("success")) is not None
        ):
            return bool(result)
        return None

    async def disable_camera(self, camera_ids: str) -> bool | None:
        """Disable camera(s) - multiple ID or single ex 1 or 1,2,3."""
        raw_data = await self._dsm.get(
            self.CAMERA_API_KEY, "Disable", {"idList": camera_ids}
        )
        if (
            isinstance(raw_data, dict)
            and (result := raw_data.get("success")) is not None
        ):
            return bool(result)
        return None

    # Snapshot
    async def capture_camera_image(
        self, camera_id: int, save: bool = True
    ) -> SurveillanceStationTakeSnapshotType | None:
        """Capture a snapshot for camera matching camera_id."""
        raw_data = await self._dsm.get(
            self.SNAPSHOT_API_KEY,
            "TakeSnapshot",
            {
                "camId": camera_id,
                "blSave": int(save),  # API requires an integer instead of a boolean
            },
        )
        if isinstance(raw_data, dict):
            return cast(SurveillanceStationTakeSnapshotType, raw_data)
        return None

    async def download_snapshot(
        self, snapshot_id: int, snapshot_size: int
    ) -> bytes | None:
        """Download snapshot image binary for a givent snapshot_id.

        Args:
            snapshot_id: ID of the snapshot we want to download.
            snapshot_size: SNAPSHOT_SIZE_ICON | SNAPSHOT_SIZE_FULL.
        """
        raw_data = await self._dsm.get(
            self.SNAPSHOT_API_KEY,
            "LoadSnapshot",
            {"id": snapshot_id, "imgSize": snapshot_size},
        )
        if isinstance(raw_data, bytes):
            return raw_data
        return None

    # Motion
    def is_motion_detection_enabled(self, camera_id: int) -> bool:
        """Return motion setting matching camera_id."""
        return bool(self._data[camera_id].is_motion_detection_enabled)

    async def enable_motion_detection(self, camera_id: int) -> dict | None:
        """Enable motion detection for camera matching camera_id."""
        raw_data = await self._dsm.get(
            self.CAMERA_EVENT_API_KEY,
            "MDParamSave",
            {"camId": camera_id, "source": MOTION_DETECTION_BY_SURVEILLANCE},
        )
        if isinstance(raw_data, dict):
            return raw_data
        return None

    async def disable_motion_detection(self, camera_id: int) -> dict | None:
        """Disable motion detection for camera matching camera_id."""
        raw_data = await self._dsm.get(
            self.CAMERA_EVENT_API_KEY,
            "MDParamSave",
            {"camId": camera_id, "source": MOTION_DETECTION_DISABLED},
        )
        if isinstance(raw_data, dict):
            return raw_data
        return None

    # Home mode
    async def get_home_mode_status(self) -> bool | None:
        """Get the state of Home Mode."""
        raw_data = await self._dsm.get(self.HOME_MODE_API_KEY, "GetInfo")
        if (
            isinstance(raw_data, dict)
            and raw_data.get("data")
            and (status := raw_data["data"].get("on")) is not None
        ):
            return bool(status)
        return None

    async def set_home_mode(self, state: bool) -> bool | None:
        """Set the state of Home Mode (state: bool)."""
        raw_data = await self._dsm.get(
            self.HOME_MODE_API_KEY, "Switch", {"on": str(state).lower()}
        )
        if (
            isinstance(raw_data, dict)
            and (result := raw_data.get("success")) is not None
        ):
            return bool(result)
        return None



"""DSM System data and actions."""

from __future__ import annotations

from typing import Any, TypedDict

from synology_dsm.api import SynoBaseApi


class SystemDataType(TypedDict):
    """Data type."""

    cpu_clock_speed: int
    cpu_cores: str
    cpu_family: str
    cpu_series: str
    enabled_ntp: bool
    ntp_server: str
    firmware_ver: str
    model: str
    ram_size: int
    serial: str
    sys_temp: int
    time: str
    time_zone: str
    time_zone_desc: str
    up_time: str
    usb_dev: list[dict[str, Any]]


class SynoCoreSystem(SynoBaseApi[SystemDataType]):
    """Class containing System data and actions."""

    API_KEY = "SYNO.Core.System"

    async def update(self) -> None:
        """Updates System data."""
        raw_data = await self._dsm.get(self.API_KEY, "info")
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data

    #
    # get information
    #
    @property
    def cpu_clock_speed(self) -> int:
        """Gets System CPU clock speed."""
        return self._data["cpu_clock_speed"]

    @property
    def cpu_cores(self) -> str:
        """Gets System CPU cores."""
        return self._data["cpu_cores"]

    @property
    def cpu_family(self) -> str:
        """Gets System CPU family."""
        return self._data["cpu_family"]

    @property
    def cpu_series(self) -> str:
        """Gets System CPU series."""
        return self._data["cpu_series"]

    @property
    def enabled_ntp(self) -> bool:
        """Gets System NTP state."""
        return self._data["enabled_ntp"]

    @property
    def ntp_server(self) -> str:
        """Gets System NTP server."""
        return self._data["ntp_server"]

    @property
    def firmware_ver(self) -> str:
        """Gets System firmware version."""
        return self._data["firmware_ver"]

    @property
    def model(self) -> str:
        """Gets System model."""
        return self._data["model"]

    @property
    def ram_size(self) -> int:
        """Gets System ram size."""
        return self._data["ram_size"]

    @property
    def serial(self) -> str:
        """Gets System serial number."""
        return self._data["serial"]

    @property
    def sys_temp(self) -> int:
        """Gets System temperature."""
        return self._data["sys_temp"]

    @property
    def time(self) -> str:
        """Gets System time."""
        return self._data["time"]

    @property
    def time_zone(self) -> str:
        """Gets System time zone."""
        return self._data["time_zone"]

    @property
    def time_zone_desc(self) -> str:
        """Gets System time zone description."""
        return self._data["time_zone_desc"]

    @property
    def up_time(self) -> str:
        """Gets System uptime."""
        return self._data["up_time"]

    @property
    def usb_dev(self) -> list:
        """Gets System connected usb devices."""
        return self._data["usb_dev"]

    #
    # do system actions
    #
    async def shutdown(self) -> None:
        """Shutdown NAS."""
        await self._dsm.get(
            self.API_KEY,
            "shutdown",
            {"local": "true"},
            max_version=1,  # shutdown method is only available on api version 1
        )

    async def reboot(self) -> None:
        """Reboot NAS."""
        await self._dsm.get(
            self.API_KEY,
            "reboot",
            max_version=1,  # reboot method is only available on api version 1
        )



"""DSM Network data."""

from __future__ import annotations

from typing import TypedDict

from synology_dsm.api import SynoBaseApi

InterfaceIp = TypedDict("InterfaceIp", {"address": str, "netmask": str})
InterfaceIpv6 = TypedDict(
    "InterfaceIpv6", {"address": str, "prefix_length": int, "scope": str}
)

NetworkInterface = TypedDict(
    "NetworkInterface",
    {
        "id": str,
        "ip": "list[InterfaceIp]",
        "ipv6": "list[InterfaceIpv6]",
        "mac": str,
        "type": str,
    },
    total=False,
)


class DsmNetworkDataType(TypedDict, total=False):
    """Data type."""

    dns: list[str]
    gateway: str
    hostname: str
    interfaces: list[NetworkInterface]
    workgroup: str


class SynoDSMNetwork(SynoBaseApi[DsmNetworkDataType]):
    """Class containing Network data."""

    API_KEY = "SYNO.DSM.Network"

    async def update(self) -> None:
        """Updates network data."""
        raw_data = await self._dsm.get(self.API_KEY, "list")
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data

    @property
    def dns(self) -> list[str]:
        """DNS of the NAS."""
        return self._data["dns"]

    @property
    def gateway(self) -> str:
        """Gateway of the NAS."""
        return self._data["gateway"]

    @property
    def hostname(self) -> str:
        """Host name of the NAS."""
        return self._data["hostname"]

    @property
    def interfaces(self) -> list[NetworkInterface]:
        """Interfaces of the NAS."""
        return self._data["interfaces"]

    def interface(self, eth_id: str) -> NetworkInterface | None:
        """Interface of the NAS."""
        for interface in self.interfaces:
            if interface["id"] == eth_id:
                return interface
        return None

    @property
    def macs(self) -> list[str]:
        """List of MACs of the NAS."""
        macs: list[str] = []
        for interface in self.interfaces:
            if (mac := interface.get("mac")) is not None:
                macs.append(mac)
        return macs

    @property
    def workgroup(self) -> str:
        """Workgroup of the NAS."""
        return self._data["workgroup"]



"""DSM Upgrade data and actions."""

from __future__ import annotations

from typing import TypedDict

from synology_dsm.api import SynoBaseApi


class UpgradeDataType(TypedDict, total=False):
    """Data type."""

    available: bool
    version: str
    version_details: dict
    reboot: str
    restart: str


class SynoCoreUpgrade(SynoBaseApi[UpgradeDataType]):
    """Class containing upgrade data and actions."""

    API_KEY = "SYNO.Core.Upgrade"
    API_SERVER_KEY = API_KEY + ".Server"

    async def update(self) -> None:
        """Updates Upgrade data."""
        raw_data = await self._dsm.get(self.API_SERVER_KEY, "check")
        if isinstance(raw_data, dict) and (data := raw_data.get("data")) is not None:
            self._data = data.get("update", data)

    @property
    def update_available(self) -> bool:
        """Gets available update info."""
        return self._data["available"]

    @property
    def available_version(self) -> str | None:
        """Gets available verion info."""
        return self._data.get("version")

    @property
    def available_version_details(self) -> dict | None:
        """Gets details about available verion."""
        return self._data.get("version_details")

    @property
    def reboot_needed(self) -> str | None:
        """Gets info if reboot is needed."""
        return self._data.get("reboot")

    @property
    def service_restarts(self) -> str | None:
        """Gets info if services are restarted."""
        return self._data.get("restart")



"""DSM 6 datas."""

from .const_6_api_auth import (
    DSM_6_AUTH_LOGIN,
    DSM_6_AUTH_LOGIN_2SA,
    DSM_6_AUTH_LOGIN_2SA_OTP,
)
from .const_6_api_info import DSM_6_API_INFO
from .core.const_6_core_security import (
    DSM_6_CORE_SECURITY,
    DSM_6_CORE_SECURITY_UPDATE_OUTOFDATE,
)
from .core.const_6_core_share import DSM_6_CORE_SHARE
from .core.const_6_core_system import (
    DSM_6_CORE_SYSTEM_DS218_PLAY,
    DSM_6_CORE_SYSTEM_DS918_PLUS,
)
from .core.const_6_core_upgrade import DSM_6_CORE_UPGRADE_FALSE, DSM_6_CORE_UPGRADE_TRUE
from .core.const_6_core_utilization import (
    DSM_6_CORE_UTILIZATION,
    DSM_6_CORE_UTILIZATION_ERROR_1055,
)
from .download_station.const_6_download_station_info import (
    DSM_6_DOWNLOAD_STATION_INFO_CONFIG,
    DSM_6_DOWNLOAD_STATION_INFO_INFO,
)
from .download_station.const_6_download_station_stat import (
    DSM_6_DOWNLOAD_STATION_STAT_INFO,
)
from .download_station.const_6_download_station_task import (
    DSM_6_DOWNLOAD_STATION_TASK_LIST,
)
from .dsm.const_6_dsm_info import DSM_6_DSM_INFORMATION
from .dsm.const_6_dsm_network import DSM_6_DSM_NETWORK_2LAN_1PPPOE
from .storage.const_6_storage_storage import (
    DSM_6_STORAGE_STORAGE_DS213_PLUS_SHR1_2DISKS_2VOLS,
    DSM_6_STORAGE_STORAGE_DS918_PLUS_RAID5_3DISKS_1VOL,
    DSM_6_STORAGE_STORAGE_DS1515_PLUS_SHR2_10DISKS_1VOL_WITH_EXPANSION,
    DSM_6_STORAGE_STORAGE_DS1819_PLUS_SHR2_8DISKS_1VOL,
)
from .surveillance_station.const_6_api_info import (
    DSM_6_API_INFO as DSM_6_API_INFO_SURVEILLANCE_STATION,
)
from .surveillance_station.const_6_surveillance_station_camera import (
    DSM_6_SURVEILLANCE_STATION_CAMERA_EVENT_MD_PARAM_SAVE,
    DSM_6_SURVEILLANCE_STATION_CAMERA_EVENT_MOTION_ENUM,
    DSM_6_SURVEILLANCE_STATION_CAMERA_GET_LIVE_VIEW_PATH,
    DSM_6_SURVEILLANCE_STATION_CAMERA_LIST,
)
from .surveillance_station.const_6_surveillance_station_home_mode import (
    DSM_6_SURVEILLANCE_STATION_HOME_MODE_GET_INFO,
    DSM_6_SURVEILLANCE_STATION_HOME_MODE_SWITCH,
)

__all__ = [
    "DSM_6_AUTH_LOGIN",
    "DSM_6_AUTH_LOGIN_2SA",
    "DSM_6_AUTH_LOGIN_2SA_OTP",
    "DSM_6_API_INFO",
    "DSM_6_CORE_SECURITY",
    "DSM_6_CORE_SECURITY_UPDATE_OUTOFDATE",
    "DSM_6_CORE_SHARE",
    "DSM_6_CORE_SYSTEM_DS218_PLAY",
    "DSM_6_CORE_SYSTEM_DS918_PLUS",
    "DSM_6_CORE_UPGRADE_FALSE",
    "DSM_6_CORE_UPGRADE_TRUE",
    "DSM_6_CORE_UTILIZATION",
    "DSM_6_CORE_UTILIZATION_ERROR_1055",
    "DSM_6_DOWNLOAD_STATION_INFO_CONFIG",
    "DSM_6_DOWNLOAD_STATION_INFO_INFO",
    "DSM_6_DOWNLOAD_STATION_STAT_INFO",
    "DSM_6_DOWNLOAD_STATION_TASK_LIST",
    "DSM_6_DSM_INFORMATION",
    "DSM_6_DSM_NETWORK_2LAN_1PPPOE",
    "DSM_6_STORAGE_STORAGE_DS1515_PLUS_SHR2_10DISKS_1VOL_WITH_EXPANSION",
    "DSM_6_STORAGE_STORAGE_DS1819_PLUS_SHR2_8DISKS_1VOL",
    "DSM_6_STORAGE_STORAGE_DS213_PLUS_SHR1_2DISKS_2VOLS",
    "DSM_6_STORAGE_STORAGE_DS918_PLUS_RAID5_3DISKS_1VOL",
    "DSM_6_API_INFO_SURVEILLANCE_STATION",
    "DSM_6_SURVEILLANCE_STATION_CAMERA_EVENT_MD_PARAM_SAVE",
    "DSM_6_SURVEILLANCE_STATION_CAMERA_EVENT_MOTION_ENUM",
    "DSM_6_SURVEILLANCE_STATION_CAMERA_GET_LIVE_VIEW_PATH",
    "DSM_6_SURVEILLANCE_STATION_CAMERA_LIST",
    "DSM_6_SURVEILLANCE_STATION_HOME_MODE_GET_INFO",
    "DSM_6_SURVEILLANCE_STATION_HOME_MODE_SWITCH",
]



"""Synology DownloadStation API wrapper."""

from __future__ import annotations

from synology_dsm.api import SynoBaseApi

from .task import SynoDownloadTask


class SynoDownloadStation(SynoBaseApi["dict[str, SynoDownloadTask]"]):
    """An implementation of a Synology DownloadStation."""

    API_KEY = "SYNO.DownloadStation.*"
    INFO_API_KEY = "SYNO.DownloadStation.Info"
    STAT_API_KEY = "SYNO.DownloadStation.Statistic"
    TASK_API_KEY = "SYNO.DownloadStation.Task"
    REQUEST_DATA = {
        "additional": "detail,file"
    }  # Can contain: detail, transfer, file, tracker, peer

    async def update(self) -> None:
        """Update tasks from API."""
        self._data = {}
        raw_data = await self._dsm.get(self.TASK_API_KEY, "List", self.REQUEST_DATA)
        if not isinstance(raw_data, dict) or (data := raw_data.get("data")) is None:
            return

        for task_data in data["tasks"]:
            if task_data["id"] in self._data:
                self._data[task_data["id"]].update(task_data)
            else:
                self._data[task_data["id"]] = SynoDownloadTask(task_data)

    # Global
    async def get_info(self) -> dict | None:
        """Return general informations about the Download Station instance."""
        raw_data = await self._dsm.get(self.INFO_API_KEY, "GetInfo")
        if isinstance(raw_data, dict):
            return raw_data
        return None

    async def get_config(self) -> dict | None:
        """Return configuration about the Download Station instance."""
        raw_data = await self._dsm.get(self.INFO_API_KEY, "GetConfig")
        if isinstance(raw_data, dict):
            return raw_data
        return None

    async def get_stat(self) -> dict | None:
        """Return statistic about the Download Station instance."""
        raw_data = await self._dsm.get(self.STAT_API_KEY, "GetInfo")
        if isinstance(raw_data, dict):
            return raw_data
        return None

    # Downloads
    def get_all_tasks(self) -> list[SynoDownloadTask]:
        """Return a list of tasks."""
        return list(self._data.values())

    def get_task(self, task_id: str) -> SynoDownloadTask | None:
        """Return task matching task_id."""
        return self._data.get(task_id)

    async def create(
        self,
        uri: str | list[str],
        unzip_password: str | None = None,
        destination: str | None = None,
    ) -> dict | None:
        """Create a new task (uri accepts HTTP/FTP/magnet/ED2K links)."""
        res = await self._dsm.post(
            self.TASK_API_KEY,
            "Create",
            {
                "uri": ",".join(uri) if isinstance(uri, list) else uri,
                "unzip_password": unzip_password,
                "destination": destination,
            },
        )
        await self.update()
        if isinstance(res, dict):
            return res
        return None

    async def pause(self, task_id: str | list[str]) -> dict | None:
        """Pause a download task."""
        res = await self._dsm.get(
            self.TASK_API_KEY,
            "Pause",
            {"id": ",".join(task_id) if isinstance(task_id, list) else task_id},
        )
        await self.update()
        if isinstance(res, dict):
            return res
        return None

    async def resume(self, task_id: str | list[str]) -> dict | None:
        """Resume a paused download task."""
        res = await self._dsm.get(
            self.TASK_API_KEY,
            "Resume",
            {"id": ",".join(task_id) if isinstance(task_id, list) else task_id},
        )
        await self.update()
        if isinstance(res, dict):
            return res
        return None

    async def delete(
        self, task_id: str | list[str], force_complete: bool = False
    ) -> dict | None:
        """Delete a download task."""
        res = await self._dsm.get(
            self.TASK_API_KEY,
            "Delete",
            {
                "id": ",".join(task_id) if isinstance(task_id, list) else task_id,
                "force_complete": force_complete,
            },
        )
        await self.update()
        if isinstance(res, dict):
            return res
        return None
